/*****************************************************************************/
/*                                                                           */
/* Scott Nichols                                                             */
/* Last modified: 11/3/2000                                                  */
/*                                                                           */
/* File: "background.c"                                                      */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#ifdef INCLUDE_JAVA
#include "GatorCameraClient.h"
#endif

#include "acticast_include.h"

/****************************** Subroutines **********************************/

void update_background(char *background, char *current, BOUNDING_BOX *boxes,
		       int num_boxes, int num_pixels)
{
  int updates_per_box;
  int i,j, xp, yp, loc;
  static unsigned int seed= 1;
  
  updates_per_box = num_pixels/num_boxes;
  srand(seed);
  
  
  
  for (i=0; i < num_boxes; i++)
    {
      for (j=0; j< updates_per_box; j++)
	{
	  /* Randomly select a pixel */
	  xp = rand() % (boxes[i].xmax - boxes[i].xmin);
	  yp = rand() % (boxes[i].ymax - boxes[i].ymin);

	  /* Get the offset */
	  loc = (yp* X_DIM + xp)*4;

	  /* Update the pixel */
	  background[loc] = current[loc];
	  background[loc+1] = current[loc+1];
	  background[loc+2] = current[loc+2];

	}
    }
  /* Change the seed */
  seed++;
}

/*****************************************************************************/

/*
  Updates background OUTSIDE the bounding boxes specified 
*/

void update_background_negative(char *background, char *current, 
				BOUNDING_BOX *boxes, 
				int num_boxes, int num_pixels)
{
  int i, xp, yp, loc, count, flag;
  
  count = 0;

  while(count < num_pixels) {
    xp = random_integer2(0, X_DIM - 1);
    yp = random_integer2(0, Y_DIM - 1);

    flag = 1;

    for(i = 0; i < num_boxes; i++) {
      if(xp >= boxes[i].xmin && xp <= boxes[i].xmax &&
	 yp >= boxes[i].ymin && yp <= boxes[i].ymax) {
	flag = -1;
      }
    }
    
    if(flag > 0) {
      loc = (yp* X_DIM + xp)*4;

      /* Update the pixel */
      background[loc] = current[loc];
      background[loc+1] = current[loc+1];
      background[loc+2] = current[loc+2];
      count++;
    }
  }
}

/*****************************************************************************/

void update_background_block(char *background, char *current, int x, int y,
			     int n, int m)
{
  int i, j, loc;
  
  
  for (i = x; i < x+n; i++)  {
    for (j = y; j < y+m; j++) {

	  loc = (j*X_DIM + i)*4;

	  background[loc] = current[loc];
	  background[loc+1] = current[loc+1];
	  background[loc+2] = current[loc+2];

	}
    }
}

/*****************************************************************************/


