/*****************************************************************************/
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* Last modified: 2/25/2001                                                  */
/*                                                                           */
/* File: "histogram.h"                                                       */
/*                                                                           */
/*****************************************************************************/

#ifndef N_HYPOTHESES_MAX
#define N_HYPOTHESES_MAX 100
#endif

/*****************************************************************************/

typedef struct one_hypothesis_leaf_struct {
  int n_objs;
  int objs[N_OBJECTS_MAX];
} HYPOTHESIS_LEAF;

typedef struct one_hypothesis_struct {
  int n;
  HYPOTHESIS_LEAF leaf[N_OBJECTS_MAX];
} ONE_HYPOTHESIS;

typedef struct hypotheses_struct {
  int n_hyp;
  ONE_HYPOTHESIS hyp[N_HYPOTHESES_MAX];
} HYPOTHESES;

/*****************************************************************************/

extern int floor_histograms(double **hist, int n_bins, int n_objects, 
			    double floor);
extern int normalize_histograms(double **hist, int n_bins, int n_objects);
extern int vector_quantize(double r, double g, double b, double **vq, int n);
extern int int_vector_quantize(int r, int g, int b, double **vq, int n);
extern int compute_histogram_class(BLOB2 *b, 
				   double **vq, int n_vq, 
				   double **hist, int n_hist);
extern int compute_histogram_prob_vec(double *p_vec,
				      BLOB2 *b, double **vq, int n_vq, 
				      double **hist, int n_hist);
extern int generate_multinomial_hypotheses(int **hyp, int n, int m);
extern int generate_hypotheses(int **hyp, int *v, int n, int m, int count);
extern double test_hypothesis(int *hyp, int n_hyp, double *p);
extern int generate_hypothesis_struct(HYPOTHESES *all, int n_objs, 
				      int n_blobs);
extern double test_one_hypothesis_struct(ONE_HYPOTHESIS *h, 
					 double **prob_table);
extern double test_one_hypothesis_leaf(HYPOTHESIS_LEAF *l, int n, 
				       double **prob_table);
extern int compute_most_consistent_hypothesis(HYPOTHESES *all, int n, int m, 
					      double **prob_table);
extern int resolve_hypothesis(ONE_HYPOTHESIS *h, BLOB2 *blobs);




/*****************************************************************************/


