/*****************************************************************************/
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* Last modified: 04/07/2001                                                 */
/*                                                                           */
/* File: "image.h"                                                           */
/*                                                                           */
/*****************************************************************************/

#ifndef MAX_BLOB_RECURSIONS
#define MAX_BLOB_RECURSIONS 100000
#endif

#ifndef MIN_BLOB_SIZE
#define MIN_BLOB_SIZE 30
#endif

/*****************************************************************************/

extern int convert_char(char c);
extern int read_ppm_file(char *file, PPM *im, int xdim, int ydim, 
			 int n_header);
extern int write_ppm_file(char *file, PPM *im);
extern int write_ppm_file_small(char *file, PPM *im);
extern int write_ppm_file_from_raw(char *file, char *im, int xdim, int ydim);
extern int pixel_value(double *p, CAMERA_MODEL *g, double *v);
extern int project_point(POINT *p, CAMERA_MODEL *camera, double *xyz);
extern int define_smoothing_filter(FILTER *f, int size);
extern int threshold(PPM *imin, PPM *imout, int t);
extern int filter(PPM *imin, PPM *imout, FILTER *f);
extern int set_dimensions(PPM *im, int xmin, int xmax, int ymin, int ymax);
extern int copy_dimensions(PPM *imin, PPM *imout);
extern int limit_dimensions(PPM *im);
extern char difference_one_third_pixel(char c2, char c1);
extern int difference(PPM *im2, PPM *im1, PPM *imout);
extern int copy_ppm_image_small(PPM *imin, PPM *imout);
extern int copy_ppm_image(PPM *imin, PPM *imout);
extern int make_single_color_image(PPM *im, int r, int g, int b);
extern int draw_big_point(PPM *im, POINT *p, int r, int g, int b);
extern int draw_point(PPM *im, POINT *p, int r, int g, int b);
extern int draw_line(PPM *im, int xmin, int xmax, int ymin, int ymax, 
		     int r, int g, int b);
extern int draw_region_box(PPM *im, int r, int g, int b);
extern int draw_ellipse(PPM *im, ELLIPSE *ell, int r, int g, int b);

extern int mask_image(PPM *imin, PPM *imout, PPM *t);
extern char hue_distance(char hc1, char hc2);
extern int convert_rgb_to_hsv(PPM *imin, PPM *imout);
extern int convert_rgb_to_hsv_blob2(BLOB2 *blobin, int blobcount);
extern int convert_rgb_to_hsv_pixel_double(double *rgb, double *hsv);
extern int convert_rgb_to_hsv_pixel(char *rgb, char *hsv);
extern double rgb_max(double r, double g, double b);
extern double rgb_min(double r, double g, double b);
extern int difference_partial(PPM *im2, PPM *im1, PPM *imout, int which,
			      char (*diff_func)(char, char));
extern int or_images(PPM *im1, PPM *im2, PPM *imout);
extern int and_images(PPM *im1, PPM *im2, PPM *imout);
extern int threshold_hsv(PPM *imin, PPM *imout, int th, int ts, int tv);
extern int border_pixel(PPM *im, int x, int y, int label);
extern int border_extraction(PPM *im, int x, int y, int label);
extern int draw_from_file(char *draw_file, PPM *im, RAW *im_raw);
extern int display_camera_model(FILE *fp, CAMERA_MODEL *c);
extern int write_camera_model(char *file_name, CAMERA_MODEL *c);
extern int read_camera_model(char *file_name, CAMERA_MODEL *c);
extern int scale_camera_model(CAMERA_MODEL *c, double scaleFactor);



/*****************************************************************************/

/* NEW BLOB2 structure */

extern int allocate_blob2(BLOB2 *b, int n_max);
extern int initialize_blob2(BLOB2 *b);
extern int convert_double_char(char *two_bytes);
extern int convert_int_to_double_char(char *two_bytes, int n);
extern int detect_blobs_ppm2(RAW2 *frame, PPM2 *t, BLOB2 *b, 
			     int min_blob_size);
extern int collect_blob_data_ppm2(RAW2 *frame, PPM2 *t, int x, int y, 
				  int label, int tst, BLOB2 *b);
extern int make_single_color_image_ppm2(PPM2 *t, int r, int g, int b);
extern int extract_blobs2_init(BLOB2 *blobs, RAW2 *im, RAW2 *ref, 
			       PPM2 *tmp1, PPM2 *tmp2, PPM2 *tmp3,
			       BOUNDING_BOX *b, int n, int threshold);
extern int extract_blobs2(BLOB2 *blobs, RAW2 *im, RAW2 *ref, 
			  PPM2 *tmp1, PPM2 *tmp2, PPM2 *tmp3,
			  BOUNDING_BOX *b, int n, int threshold);
extern int display_blob2(FILE *fp, BLOB2 *b);
extern int merge_blobs(BLOB2 *out, BLOB2 *blist, int n);

/*****************************************************************************/

extern int compute_bounding_boxes_from_blobs(BLOB2 *blobs, int n_blobs,
					     BOUNDING_BOX *boxes, 
					     int expand_constant);

extern int draw_big_point_from_double(PPM *im, DPOINT *p, int r, int g, int b);
extern int draw_line_from_double(PPM *im, DPOINT *p1, DPOINT *p2,
				 int r, int g, int b);

/*****************************************************************************/

extern int qrdcmp(double **a, int n, double *c, double *d);
extern int convert_householder_to_q(double **q, int n, double *u, double c);
extern int qr_decomposition(double **a, int n, double **q, double **r);
extern int intrinsic_extrinsic_decomposition(CAMERA_MODEL *c, 
					     INTRINSIC_EXTRINSIC *d);
extern int display_intrinsic_extrinsic(FILE *fp, INTRINSIC_EXTRINSIC *d);

/*****************************************************************************/

extern int draw_convex_hull(PPM *im, DPOINT *pts, int n_pts, DPOINT *ch, 
			    int n_ch);
extern int draw_big_point_from_double_raw(char *im, DPOINT *p, 
					  int r, int g, int b);
extern int draw_line_from_double_raw(char *im, DPOINT *p1, DPOINT *p2,
				     int r, int g, int b);
extern int draw_convex_hull_raw(char *im, DPOINT *pts, int n_pts, 
				DPOINT *ch, int n_ch);


