/*****************************************************************************/
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* Last modified: 03/02/2001                                                 */
/*                                                                           */
/* File: "image_struct.h"                                                    */
/*                                                                           */
/*****************************************************************************/

#define POINT_TYPE 1
#define BIG_POINT_TYPE 2
#define LINE_TYPE 3
#define REGION_BOX_TYPE 4
#define ELLIPSE_TYPE 5

#ifndef N_OBJECTS_MAX
#define N_OBJECTS_MAX 5
#endif

#ifndef N_BLOBS_MAX
#define N_BLOBS_MAX 500
#endif

#ifndef X_DIM
#define X_DIM 640
#endif

#ifndef Y_DIM
#define Y_DIM 480
#endif

#ifndef X_DIM_MAX
#define X_DIM_MAX 640
#endif

#ifndef Y_DIM_MAX
#define Y_DIM_MAX 480
#endif

#ifndef N_HEADER
#define N_HEADER 15
#endif

#define RED 0   // for indexing
#define GREEN 1 // for indexing
#define BLUE 2  // for indexing

// pre-defined colors

#define COLOR_WHITE 255, 255, 255
#define COLOR_YELLOW 255, 255, 0
#define COLOR_MAGENTA 255, 0, 255
#define COLOR_CYAN 0, 255, 255
#define COLOR_RED 255, 0, 0
#define COLOR_GREEN 0, 255, 0
#define COLOR_BLUE 0, 0, 255
#define COLOR_GRAY 128, 128, 128

/*****************************************************************************/

/* Basic camera model structure */

#ifndef N_CAMERA_PARAMETERS
#define N_CAMERA_PARAMETERS 14
#endif

typedef struct camera_model_struct {
  double f;
  double s[3][4];
} CAMERA_MODEL;

/*****************************************************************************/

typedef struct qr_decompose_struct {
  double r[3][3];
  double k[3][3];
  double t[3];
} INTRINSIC_EXTRINSIC;

/*****************************************************************************/

/* Basic image structures */

typedef struct ppm_image_struct {
  char p[X_DIM_MAX][Y_DIM_MAX][3];
  int xdim;
  int ydim;
  int xmin;
  int xmax;
  int ymin;
  int ymax;
} PPM;

typedef struct ppm_raw_struct {
  char *p;
  int xdim;
  int ydim;
  int xmin;
  int xmax;
  int ymin;
  int ymax;
} RAW;

/*****************************************************************************/

/* Basic filter structure */

#ifndef FILTER_MAX
#define FILTER_MAX 20
#endif

typedef struct filter_struct {
  double m[FILTER_MAX][FILTER_MAX];
  int size;
} FILTER;

/*****************************************************************************/

/* Advanced image structures */

#define MAX_MASK_COORD 200 // maximum number of range points = MAX_MASK_COORD/2
#define MASK_START 0      // actual value is important, do not change
#define MASK_END 1        // actual value is important, do not change

typedef struct mask_coord_struct {
  int x;
  int type;
} MASK_COORD;

typedef struct mask_line_struct {
  int n;
  MASK_COORD c[MAX_MASK_COORD];
} MASK_LINE;

typedef struct mask_struct {
  MASK_LINE line[Y_DIM];
} MASK;

typedef struct ppm2_struct {
  PPM im;
  MASK m;
} PPM2;

typedef struct raw2_struct {
  RAW im;
  MASK m;
} RAW2;

/*****************************************************************************/

/* Basic pixel POINT structures */

typedef struct point_struct {
  int x;
  int y;
} POINT;

typedef struct dpoint_struct {
  double x;
  double y;
} DPOINT;

/*****************************************************************************/

/* General ELLIPSE_2D structure */

typedef struct ellipse_2d_struct {
  double cov[2][2];
  double mean[2];
  double major_axis[2];
  double ev[2];
  double theta;
  double a, b;
  int xmin, ymin, xmax, ymax; // New
} ELLIPSE;


/*****************************************************************************/

/* General BLOB2 structure */

#ifndef BLOB2_MAX_SIZE
#define BLOB2_MAX_SIZE 1000
#endif

typedef struct blob2_struct {
  int label;
  int n;
  POINT first;
  double **rgb_t; // transpose(rgb)
  double **xy_t;  // transpose(xy)
  ELLIPSE ell;    // blob position statistics
  int *type;      // multi-purpose integer label vector (for classification)
  double *val;    // multi-purpose real value vector    (for classification)
  int tags[N_OBJECTS_MAX];
  int n_tags;
} BLOB2;

/*****************************************************************************/
