/*****************************************************************************/
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* Last modified: 03/02/2001                                                 */
/*                                                                           */
/* File: "mask.c"                                                            */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#ifdef INCLUDE_JAVA
#include "GatorCameraClient.h"
#endif

#include "acticast_include.h"

/****************************** Subroutines **********************************/

int convert_ppm_mask_to_mask_struct(PPM *im, MASK *m)
{
  int i, j, flag, counter, tst, n = 0;
  
  clear_mask(m);
  
  for(i = 0; i < Y_DIM; i++) {
    flag = 0;
    counter = 0;
    for(j = 0; j < X_DIM; j++) {
      tst = convert_char(im->p[j][i][0]);
      if(tst == 255 && flag == 0) {
	m->line[i].c[counter].x = j;
	m->line[i].c[counter].type = MASK_START;
	counter++;
	flag = 1;
	n++;
      } else if (tst == 0 && flag == 1) {
	m->line[i].c[counter].x = j;
	m->line[i].c[counter].type = MASK_END;
	counter++;
	flag = 0;
      } else if (flag == 1 && j < X_DIM-1) {
	n++;
      } else if (flag == 1 && j == X_DIM - 1) {
	m->line[i].c[counter].x = j;
	m->line[i].c[counter].type = MASK_END;
	counter++;
	flag = 0;
      }
    }
    m->line[i].n = counter;
  }
  
  return(n);
}

/*****************************************************************************/

int read_mask_file(char *file, MASK *m)
{
  FILE *fp;
  int i, y;

  clear_mask(m);

  fp = fopen(file, "r");

  while(fscanf(fp, "%d", &y) != EOF) {
    fscanf(fp, "%d", &m->line[y].n);
    for(i = 0; i < m->line[y].n; i++) {
      fscanf(fp, "%d", &m->line[y].c[i].x);
      fscanf(fp, "%d", &m->line[y].c[i].type);
    }
  }

  close(fp);

  return(1);
}

/*****************************************************************************/

int write_mask_file(char *file, MASK *m)
{
  FILE *fp;

  fp = fopen(file, "w");
  print_mask(fp, m);
  close(fp);

  return(1);
}

/*****************************************************************************/

int read_search_region_file(char *file, BOUNDING_BOX *b)
{
  FILE *fp;
  int tmp, count = 0;

  file_no_exist(file, "initial search region file");

  fp = fopen(file, "r");

  while(fscanf(fp, "%d", &tmp) != EOF) {
    b[count].xmin = tmp;
    fscanf(fp, "%d", &b[count].ymin);
    fscanf(fp, "%d", &b[count].xmax);
    fscanf(fp, "%d", &b[count].ymax);
    count++;
  }

  fclose(fp);

  return(count);
}

/*****************************************************************************/

/* purposely didn't limit by dimensions of image */

int change_bounding_box(BOUNDING_BOX *b, int n)
{
  b->xmin = b->xmin-n;
  b->xmax = b->xmax+n;

  b->ymin = b->ymin-n;
  b->ymax = b->ymax+n;

  return(1);
}
/*****************************************************************************/

int dilate_bounding_box(BOUNDING_BOX *b)
{
  return(change_bounding_box(b, 1));
}

/*****************************************************************************/

int dilate_bounding_boxes(BOUNDING_BOX *b, int n_b)
{
  int i; 
  
  for(i = 0; i < n_b; i++)
    dilate_bounding_box(&b[i]);
  
  return(1);
}

/*****************************************************************************/

int contract_bounding_box(BOUNDING_BOX *b)
{
  return(change_bounding_box(b,-1));
}


/*****************************************************************************/

int contract_bounding_boxes(BOUNDING_BOX *b, int n_b)
{
  int i; 
  
  for(i = 0; i < n_b; i++)
    contract_bounding_box(&b[i]);

  return(1);
}

/*****************************************************************************/

int clear_mask(MASK *m)
{
  int i;

  for(i = 0; i < Y_DIM; i++)
    m->line[i].n = 0;

  return(1);
}

/*****************************************************************************/

/* quick insertion sort from Numerical Recipes (short lists) */

int sort_mask_coord(MASK_COORD *v, int n)
{
  int i, j, x, type, len;

  for(j = 1; j < n; j++) {
    x = v[j].x;
    type = v[j].type;
    i = j - 1;
    while(i >= 0 && v[i].x > x) {
      v[i+1].x = v[i].x;
      v[i+1].type = v[i].type;
      i--;
    }
    v[i+1].x = x;
    v[i+1].type = type;
  }

  /* check for ties */

  len = n;

  for(j = 0; j < len-1; j++) {
    if(v[j].x == v[j+1].x) {
      for(i = j; i < len-1; i++) {
	v[i].x = v[i+1].x;
	v[i].type = v[i+1].type;
      }
      len--;
    }
  }

  return(len);
}

/*****************************************************************************/

int check_overlap(MASK_COORD *out, MASK_COORD *a, MASK_COORD *b)
{
  int al, ah, bl, bh;

  al = a[0].x;
  ah = a[1].x;

  bl = b[0].x;
  bh = b[1].x;

  //fprintf(stdout, "al = %d, ah = %d, bl = %d, bh = %d\n", al, ah, bl, bh);
  if(ah >= bl && ah <= bh && al <= bl) {
    out[0].x = al;
    out[1].x = bh;
    return(1);
  } else if (al >= bl && al <= bh && ah >= bl && ah <= bh) {
    out[0].x = bl;
    out[1].x = bh;
    return(2);
  } else if (al >= bl && al <= bh && ah >= bh) {
    out[0].x = bl;
    out[1].x = ah;
    return(3);
  } else if(bl >= al && bl <= ah && bh >= al && bh <= ah) {
    out[0].x = al;
    out[1].x = ah;
    return(4);
  }

  out[0].x = 0;
  out[1].x = 0;
  return(0);
}

/*****************************************************************************/

int prep_mask_line(MASK_LINE *l)
{
  int i, j, k, tst;
  MASK_COORD out[2];
  

  for(i = 0; i < l->n; i+=2)
    for(j = i+2; j < l->n; j+=2) {
      tst = check_overlap(&out[0], &l->c[i], &l->c[j]);
      //fprintf(stdout, "(%d, %d): tst = %d --> (%d, %d)\n", i/2, j/2, tst,
      //out[0].x, out[1].x);
      if(tst > 0) {
	l->c[i].x = out[0].x;
	l->c[i+1].x = out[1].x;
	
	for(k = j; k < l->n - 2; k+=2) {
	  l->c[k].x = l->c[k+2].x;
	  l->c[k+1].x = l->c[k+3].x;
	}
	return(l->n - 2);
      }
    }
  
  return(l->n);
}
/*****************************************************************************/


int fix_mask_line(MASK_LINE *l)
{
  int tst, done = 0;

  while(!done) {
    tst = prep_mask_line(l);
    if(tst != l->n) {
      l->n = tst;
    } else {
      done = 1;
    }
  }
   
  return(1);
}

/*****************************************************************************/

int check_mask_coordinates(MASK_LINE *l, int index)
{
  int i, count;

  if(l->c[index].type == l->c[index+1].type) {
    for(i = index+(2-l->c[index].type); i < l->n; i++) {
      l->c[i-1].x = l->c[i].x;
      l->c[i-1].type = l->c[i].type;
    }
    l->n--;
    count = 0;
  } else {
    count = 1;
  }

  return(count);
}

/*****************************************************************************/

int create_mask(MASK *m, BOUNDING_BOX *b, int n_b)
{
  int i, j, n;
  int xmin, ymin, xmax, ymax;

  clear_mask(m);
  
  for(i = 0; i < n_b; i++) {

#ifndef SUPPRESS_ALL_OUTPUT
#ifdef LEVEL_THREE_DIAGNOSTIC
    fprintf(stderr, "Region box #%d: (%d, %d) to (%d, %d)\n", i+1,
	    b[i].xmin, b[i].ymin, b[i].xmax, b[i].ymax);
#endif
#endif

    xmin = limit_int(b[i].xmin, 0, X_DIM-1);
    ymin = limit_int(b[i].ymin, 0, Y_DIM-1);
    xmax = limit_int(b[i].xmax, 0, X_DIM-1);
    ymax = limit_int(b[i].ymax, 0, Y_DIM-1);

    for(j = ymin; j <= ymax; j++) {
      n = m->line[j].n;
      m->line[j].c[n].x = xmin;
      m->line[j].c[n].type = MASK_START;
      m->line[j].c[n+1].x = xmax;
      m->line[j].c[n+1].type = MASK_END;
      m->line[j].n += 2;
    }
  }
  
  //fprintf(stdout, "Before:\n");
  //print_mask(stdout, m);
  for(i = 0; i < Y_DIM; i++) {
    if(m->line[i].n > 2) {
      fix_mask_line(&m->line[i]);
    }
  }
  //fprintf(stdout, "After:\n");
  //print_mask(stdout, m);

  return(1);
}

/*****************************************************************************/

int print_mask(FILE *fp, MASK *m)
{
  int i, j, n;

  for(i = 0; i < Y_DIM; i++) {
    n = m->line[i].n;
    if(n != 0) {
      fprintf(fp, "%d %d ", i, n);
      for(j = 0; j < n; j++) {
	fprintf(fp, "%d %d ", 
		m->line[i].c[j].x, m->line[i].c[j].type);
      }
      
      fprintf(fp, "\n");
    }
  }
  
  return(1);
}

/*****************************************************************************/

int draw_mask(PPM *im, MASK *m)
{
  int i, j, n;
  
  for(i = 0; i < Y_DIM; i++) {
    n = m->line[i].n;
    for(j = 0; j < n; j+=2)
      draw_line(im, 
		m->line[i].c[j].x, m->line[i].c[j+1].x, i, i, 255, 255,255);
  }
  
  return(1);
}

/*****************************************************************************/

int copy_masks(MASK *in, MASK *out)
{
  int i, j;

  for(i = 0; i < Y_DIM; i++) {
    out->line[i].n = in->line[i].n;
    for(j = 0; j < in->line[i].n; j++) {
      out->line[i].c[j].x = in->line[i].c[j].x;
      out->line[i].c[j].type = in->line[i].c[j].type;
    }
  }

  return(1);
}

/*****************************************************************************/

int copy_mask_raw2_to_ppm2(RAW2 *in, PPM2 *out)
{
  int i, j;
  copy_dimensions_raw_to_ppm(&in->im, &out->im);
  copy_masks(&in->m, &out->m);
  return(1);
}

/*****************************************************************************/

int copy_mask_raw2_to_raw2(RAW2 *in, RAW2 *out)
{
  int i, j;
  copy_dimensions_raw_to_raw(&in->im, &out->im);
  copy_masks(&in->m, &out->m);
  return(1);
}

/*****************************************************************************/

int copy_mask_ppm2_to_ppm2(PPM2 *in, PPM2 *out)
{
  int i, j;
  copy_dimensions(&in->im, &out->im);
  copy_masks(&in->m, &out->m);

  return(1);
}


/*****************************************************************************/

int difference_hsv_raw2(RAW2 *im2, RAW2 *im1, PPM2 *imout)
{
  int i, j, j1, loc;
  char *buf1_rgb, *buf2_rgb, *buf1_hsv, *buf2_hsv;
  
  copy_mask_raw2_to_ppm2(im1, imout);
  
  for(i = 0; i < im1->im.ydim; i++) {
    for(j1 = 0; j1 < im1->m.line[i].n; j1+=2) {
      for(j = im1->m.line[i].c[j1].x; j <= im1->m.line[i].c[j1+1].x; j++) {
	loc = (i*im1->im.xdim + j)*4;
	buf1_rgb = &im1->im.p[loc];
	buf2_rgb = &im2->im.p[loc];
	
	convert_rgb_to_hsv_pixel(&buf1_rgb[0], &buf1_hsv[0]);
	convert_rgb_to_hsv_pixel(&buf2_rgb[0], &buf2_hsv[0]);

	imout->im.p[j][i][0] = hue_distance(buf2_hsv[2], buf1_hsv[2]);
	imout->im.p[j][i][1] = difference_one_third_pixel(buf2_hsv[1], buf1_hsv[1]);
	imout->im.p[j][i][2] = difference_one_third_pixel(buf2_hsv[0], buf1_hsv[0]);
      }
    }
  }
  
  return(1);
}

/*****************************************************************************/

int difference_raw2(RAW2 *im2, RAW2 *im1, PPM2 *imout)
{
  int i, j, j1, loc;
  char *buf1, *buf2;
  
  copy_mask_raw2_to_ppm2(im1, imout);
  
  for(i = 0; i < im1->im.ydim; i++) {
    for(j1 = 0; j1 < im1->m.line[i].n; j1+=2) {
      for(j = im1->m.line[i].c[j1].x; j <= im1->m.line[i].c[j1+1].x; j++) {
	loc = (i*im1->im.xdim + j)*4;
	buf1 = &im1->im.p[loc];
	buf2 = &im2->im.p[loc];
	
	imout->im.p[j][i][0] = difference_one_third_pixel(buf2[2], buf1[2]);
	imout->im.p[j][i][1] = difference_one_third_pixel(buf2[1], buf1[1]);
	imout->im.p[j][i][2] = difference_one_third_pixel(buf2[0], buf1[0]);
      }
    }
  }
  
  return(1);
}

/*****************************************************************************/

int difference_ppm2(PPM2 *im2, PPM2 *im1, PPM2 *imout)
{
  int i, j, j1;
  
  copy_mask_ppm2_to_ppm2(im1, imout);
  
  for(i = 0; i < im1->im.ydim; i++) {
    for(j1 = 0; j1 < im1->m.line[i].n; j1+=2) {
      for(j = im1->m.line[i].c[j1].x; j <= im1->m.line[i].c[j1+1].x; j++) {
	imout->im.p[j][i][0] = 
	  difference_one_third_pixel(im2->im.p[j][i][0], im1->im.p[j][i][0]);
	imout->im.p[j][i][1] = 
	  difference_one_third_pixel(im2->im.p[j][i][1], im1->im.p[j][i][1]);
	imout->im.p[j][i][2] = 
	  difference_one_third_pixel(im2->im.p[j][i][2], im1->im.p[j][i][2]);
      }
    }
  }
  
  return(1);
}

/*****************************************************************************/

int threshold_ppm2(PPM2 *in, PPM2 *out, int t)
{
  int i, j, j1, k, tmp, v;

  limit_dimensions(&in->im);
  copy_mask_ppm2_to_ppm2(in, out);
  
  for(i = 0; i < in->im.ydim; i++) {
    for(j1 = 0; j1 < in->m.line[i].n; j1+=2) {
      for(j = in->m.line[i].c[j1].x; j <= in->m.line[i].c[j1+1].x; j++) {
	tmp = 0;
	for(k = 0; k < 3; k++) {
	  tmp += convert_char(in->im.p[j][i][k]);
	}
	tmp /= 3;
	if(tmp < t) 
	  v = 0;
	else 
	  v = 255;
	
	out->im.p[j][i][0] = (char) v;
	out->im.p[j][i][1] = (char) v;
	out->im.p[j][i][2] = (char) v;
      }
    }
  }
  
  return(1);
}

/*****************************************************************************/
