/*****************************************************************************/
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* Last modified: 03/25/2001                                                 */
/*                                                                           */
/* File: "xdisplay.c"                                                        */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>

#include "matrix.h"
#include "image_struct.h"
#include "geometry.h"
#include "xdisplay.h"
//#include "diagnostic.h"

/*****************************************************************************/

int open_xdisplay(char *im)
{
  XEvent e;
  int black, white;

  if(xdisplay_X_DIM <= 0) {
    xdisplay_X_DIM = X_DIM;
    xdisplay_Y_DIM = Y_DIM;
    xdisplay_Blowup_Factor = 1;
  } else {
    xdisplay_Blowup_Factor = xdisplay_X_DIM/X_DIM;
    if(xdisplay_Blowup_Factor > 1)
      xdisplay_Blowup_Buffer = 
	(char *) malloc(sizeof(char)*xdisplay_X_DIM*xdisplay_Y_DIM*4);
  }

  xdisplay_Display = XOpenDisplay(0);
  assert(xdisplay_Display);

  black = BlackPixel(xdisplay_Display, DefaultScreen(xdisplay_Display));
  white = WhitePixel(xdisplay_Display, DefaultScreen(xdisplay_Display));

  xdisplay_Window = 
    XCreateSimpleWindow(xdisplay_Display, 
			DefaultRootWindow(xdisplay_Display), 
			0, 0, xdisplay_X_DIM+1, xdisplay_Y_DIM+1, 0, 
			black, black);

  XSelectInput(xdisplay_Display, xdisplay_Window, StructureNotifyMask);
  XMapWindow(xdisplay_Display, xdisplay_Window);
  xdisplay_GC = XCreateGC(xdisplay_Display, xdisplay_Window, 0, 0);
  XSetForeground(xdisplay_Display, xdisplay_GC, white);
 
  for(;;) {
    XNextEvent(xdisplay_Display, &e);
    if (e.type == MapNotify)
      break;
  }

  xdisplay_XImage = 
    XGetImage(xdisplay_Display, xdisplay_Window, 1, 1, 
	      xdisplay_X_DIM, xdisplay_Y_DIM, 255,
	      ZPixmap);
  XPutImage(xdisplay_Display, xdisplay_Window, xdisplay_GC, 
	    xdisplay_XImage, 1, 1, 1, 1, xdisplay_X_DIM, xdisplay_Y_DIM);
  XFlush(xdisplay_Display);

  full_update_xdisplay(im);

  return(1);
}

/*****************************************************************************/

int update_xdisplay(char *im, 
		    BOUNDING_BOX *prev, int n_prev, 
		    BOUNDING_BOX *curr, int n_curr)
{		 
  int i, w, h;


  if(prev == NULL || curr == NULL) {
    full_update_xdisplay(im);
    return(1);
  }

  xdisplay_XImage->data = im;
  
  for(i = 0; i < n_prev; i++) {
    w = prev[i].xmax - prev[i].xmin + 1;
    h = prev[i].ymax - prev[i].ymin + 1;
    XPutImage(xdisplay_Display, xdisplay_Window, xdisplay_GC, 
	      xdisplay_XImage,
	      prev[i].xmin, prev[i].ymin, prev[i].xmin, prev[i].ymin, w, h);
  }
  
  for(i = 0; i < n_curr; i++) {
    w = curr[i].xmax - curr[i].xmin + 1;
    h = curr[i].ymax - curr[i].ymin + 1;
    XPutImage(xdisplay_Display, xdisplay_Window, xdisplay_GC, 
	      xdisplay_XImage,
	      curr[i].xmin, curr[i].ymin, curr[i].xmin, curr[i].ymin, w, h);
  }

  return(1);
}

/*****************************************************************************/

int full_update_xdisplay(char *im)
{
  if(im != NULL) {
    if(xdisplay_Blowup_Factor == 1) {
      xdisplay_XImage->data = im;
    } else {
      blowup_xdisplay_image(xdisplay_Blowup_Buffer, im);
      xdisplay_XImage->data = xdisplay_Blowup_Buffer;
    }

    XPutImage(xdisplay_Display, xdisplay_Window, xdisplay_GC, 
	      xdisplay_XImage,
	      1, 1, 1, 1, xdisplay_X_DIM+1, xdisplay_Y_DIM+1);
  }

  return(1);
}

/*****************************************************************************/

int copy_bounding_box_to_char(char *vid, char *im, BOUNDING_BOX *b)
{
  char *buf;
  int i, j, loc, count = 0, int_length;

  int_length = sizeof(int);

  b->xmax = limit_int(b->xmax, 0, X_DIM-1);
  b->ymax = limit_int(b->ymax, 0, Y_DIM-1);

  memcpy(&vid[count], &b->xmin, int_length); count += int_length;
  memcpy(&vid[count], &b->ymin, int_length); count += int_length;
  memcpy(&vid[count], &b->xmax, int_length); count += int_length;
  memcpy(&vid[count], &b->ymax, int_length); count += int_length;


  for(i = b->ymin; i <= b->ymax; i++) {
    for(j = b->xmin; j <= b->xmax; j++) {
      loc = (i*X_DIM + j)*4;
      buf = &im[loc];
      vid[count++] = buf[0];
      vid[count++] = buf[1];
      vid[count++] = buf[2];
    }
  }

  return(count);
}

/*****************************************************************************/

int copy_char_to_bounding_box(char *vid, char *im, BOUNDING_BOX *b)
{
  int i, j, loc, count = 0, int_length;

  int_length = sizeof(int);

  memcpy(&b->xmin, &vid[count], int_length); count+=int_length;
  memcpy(&b->ymin, &vid[count], int_length); count+=int_length;
  memcpy(&b->xmax, &vid[count], int_length); count+=int_length;
  memcpy(&b->ymax, &vid[count], int_length); count+=int_length;
    
  //fprintf(stderr, "Bounding box = (%d, %d) to (%d, %d)\n",
  //b->xmin, b->ymin, b->xmax, b->ymax);

  for(i = b->ymin; i <= b->ymax; i++) {
    for(j = b->xmin; j <= b->xmax; j++) {
      loc = (i*X_DIM + j)*4;
      im[loc]   = vid[count++];
      im[loc+1] = vid[count++];
      im[loc+2] = vid[count++];
      im[loc+3] = (char) 0;
    }
  }

  return(count);
}

/*****************************************************************************/

int compute_size_of_bounding_box(BOUNDING_BOX *b)
{
  int w, h, count;

  b->xmax = limit_int(b->xmax, 0, X_DIM-1);
  b->ymax = limit_int(b->ymax, 0, Y_DIM-1);

  w = b->xmax - b->xmin + 1;
  h = b->ymax - b->ymin + 1;
  count = (w*h*3)+sizeof(int)*4;

  return(count);
}

/*****************************************************************************/

int write_compressed_video_file(char *file, char *vid, int n_vid)
{
  int i, j;
  FILE *fp;

  fp = fopen(file, "w");
  fwrite(vid, n_vid, 1, fp);
  fclose(fp);

  /*
  for(i = 0; i < n_vid; i++)
    fprintf(fp, "%c", vid[i]);
  */

  return(1);
}

/*****************************************************************************/

char *read_compressed_video_file(char *file, char *vid, int *alloc)
{
  int count = 0;
  FILE *fp;

  if(*alloc == 0) {
    *alloc = file_size(file);
    vid = (char *) malloc(sizeof(char)*(*alloc));
    assert(vid);
  }

  fp = fopen(file, "r");
  fread(vid, *alloc, 1, fp);
  fclose(fp);

  /*
  while(fscanf(fp, "%c", &vid[count]) != EOF) {
    count++;
  }
  */

  return(vid);
}

/*****************************************************************************/

int play_compressed_video(char *vid, int n_vid, char *ref, char *curr,
			  long delay, int decode, int erase)
{
  BOUNDING_BOX prev[N_BLOBS_MAX];
  int i, n_prev = 0, vid_pointer = 0;
  double ideal_time, time_diff_d, delay_d;
  struct timeval base_time, curr_time, time_diff;
  struct timezone tz;
  long actual_delay;

  system("date");
  gettimeofday(&base_time, &tz);

  while(vid_pointer < n_vid) {
    vid_pointer =
      play_compressed_video_one_frame(ref, curr, vid, vid_pointer, 
				      &prev[0], &n_prev, decode, erase);
    gettimeofday(&curr_time, &tz);
    
    time_diff.tv_sec = curr_time.tv_sec - base_time.tv_sec;
    time_diff.tv_usec = curr_time.tv_usec - base_time.tv_usec;
    if(time_diff.tv_usec < 0) {
      time_diff.tv_sec--;
      time_diff.tv_usec += 1000000;
    }
    time_diff_d = ((double) time_diff.tv_sec) + 
      (((double) time_diff.tv_usec)/1000000.);
    ideal_time = ((double) (xdisplay_FrameCounter - 1000))*((double) delay)/
      1000000.;

    delay_d = ideal_time - time_diff_d;
    if(delay_d > 0)
      actual_delay = (long) (delay_d * 1000000.);
    else 
      actual_delay = 0;

    //fprintf(stdout, 
    //    "#%d: %.6f - %.6f = %.6f [ %ld ]\n",
    //    xdisplay_FrameCounter, 
    //    ideal_time, time_diff_d, delay_d, actual_delay);
    

    usleep(actual_delay);
  }
  system("date");

  return(1);
}

/*****************************************************************************/

int play_compressed_video_one_frame(char *im0, char *im1, 
				    char *vid, int vid_pointer,
				    BOUNDING_BOX *prev, int *n_prev,
				    int decode, int erase)
{
  int i, n, int_length;
  BOUNDING_BOX b;
  char str[256];
  char *tmp_ptr;

  int_length = sizeof(int);

  if(erase > 0) {
    for(i = 0; i < *n_prev; i++) {
      update_xdisplay_one_box(im0, &prev[i]);
    }
  }

  memcpy(&n, &vid[vid_pointer], int_length); vid_pointer+=int_length;
  //fprintf(stderr, "N = %d (vid_pointer = %d)\n", n, vid_pointer);

  if(decode != 0) {
    memcpy(im1, xdisplay_background, X_DIM*Y_DIM*4);
  }


  for(i = 0; i < n; i++) {
    vid_pointer += copy_char_to_bounding_box(&vid[vid_pointer], im1, &b);
    update_xdisplay_one_box(im1, &b);
    prev[i].xmin = b.xmin;
    prev[i].xmax = b.xmax;
    prev[i].ymin = b.ymin;
    prev[i].ymax = b.ymax;
  }

  if(xdisplay_FrameShow > 0) {
    sprintf(&str[0], "%d", xdisplay_FrameCounter);
    tmp_ptr = xdisplay_XImage->data;
    xdisplay_XImage->data = xdisplay_blank;
    XPutImage(xdisplay_Display, xdisplay_Window, xdisplay_GC, 
	      xdisplay_XImage, 0, 0, 0, 0, FRAME_LABEL_WIDTH, 
	      FRAME_LABEL_HEIGHT);
    xdisplay_XImage->data = tmp_ptr;
    XDrawString(xdisplay_Display, xdisplay_Window, xdisplay_GC, 
		FRAME_LABEL_X_OFFSET, FRAME_LABEL_Y_OFFSET,
		&str[0], strlen(&str[0]));
  }

  *n_prev = n;

  if(decode != 0) {
    if((xdisplay_FrameCounter >= xdisplay_FrameMin &&
       xdisplay_FrameCounter <= xdisplay_FrameMax) ||
       (xdisplay_FrameMin < 0 || xdisplay_FrameMax < 0)) {
      sprintf(&str[0], "%s%d.ppm", &xdisplay_OutputFile[0],
	      xdisplay_FrameCounter);
      write_ppm_file_from_raw(&str[0], xdisplay_XImage->data, X_DIM, Y_DIM);
      fprintf(stderr, "OutFrame --> %s.\n", &str[0]);
    }
  }
  xdisplay_FrameCounter++;
  
  return(vid_pointer);
}

/*****************************************************************************/

int update_xdisplay_one_box(char *im, BOUNDING_BOX *b)
{
  if(xdisplay_X_DIM == X_DIM) {
    xdisplay_XImage->data = im;
    XPutImage(xdisplay_Display, xdisplay_Window, xdisplay_GC, xdisplay_XImage, 
	      b->xmin, b->ymin, b->xmin, b->ymin, 
	      b->xmax-b->xmin+1,
	      b->ymax-b->ymin+1);
  } else {
    blowup_xdisplay_image(xdisplay_Blowup_Buffer, im);
    xdisplay_XImage->data = xdisplay_Blowup_Buffer;
    XPutImage(xdisplay_Display, xdisplay_Window, xdisplay_GC, xdisplay_XImage, 
	      b->xmin*xdisplay_Blowup_Factor, 
	      b->ymin*xdisplay_Blowup_Factor, 
	      b->xmin*xdisplay_Blowup_Factor,
	      b->ymin*xdisplay_Blowup_Factor,
	      (b->xmax-b->xmin+1)*xdisplay_Blowup_Factor,
	      (b->ymax-b->ymin+1)*xdisplay_Blowup_Factor);
  }

  return(1);
}

/*****************************************************************************/

int blowup_xdisplay_image(char *out, char *in)
{
  int i, j, loc, blowup;
  char *buf;

  for(i = 0; i < Y_DIM; i++)
    for(j = 0; j < X_DIM; j++) {
      loc = (i*X_DIM + j)*4;
      buf = &in[loc];
      put_one_expanded_pixel(out, buf, j, i);
    }

  return(1);
}

/*****************************************************************************/

int put_one_expanded_pixel(char *out, char *buf, int x, int y)
{
  int i, j, loc;
  int blowup;

  blowup = xdisplay_Blowup_Factor;

  for(i = 0; i < blowup; i++) {
    for(j = 0; j < blowup; j++) {
      loc = ((y*blowup + i)*xdisplay_X_DIM + (x*blowup + j))*4;
      out[loc] = buf[0];
      out[loc+1] = buf[1];
      out[loc+2] = buf[2];
      out[loc+3] = buf[3];
    }
  }

  return(1);
}

/*****************************************************************************/

char *initialize_video_buffer(char *vid, int size)
{
  vid = (char *) malloc(sizeof(char)*size);
  assert(vid);

  return(vid);
}

/*****************************************************************************/

#ifndef RECORD_VIDEO_FILE
#define RECORD_VIDEO_FILE "video.dat"
#endif

int initialize_xdisplay_video_buffer(int size)
{
  xdisplay_video = 
    initialize_video_buffer(xdisplay_video, size);
  xdisplay_video_buffer_size = size;
  xdisplay_video_pointer = 0;
  sprintf(&xdisplay_VideoOutputFile[0], RECORD_VIDEO_FILE);

  return(1);
}

/*****************************************************************************/

int update_xdisplay_video_buffer(char *frame, BOUNDING_BOX *b, int n)
{
  int tst;

  tst = update_video_buffer(xdisplay_video, 
			    xdisplay_video_pointer,
			    xdisplay_video_buffer_size,
			    frame, b, n);
  if(tst > 0) {
    xdisplay_video_pointer = tst;
  } else {
    fprintf(stderr, "Video data is about to exceed maximum buffer size.\n");
    dump_xdisplay_video_buffer(&xdisplay_VideoOutputFile[0]);
    exit(1);
  }

  return(1);
}

/*****************************************************************************/

int dump_xdisplay_video_buffer(char *file)
{
    fprintf(stderr, "Dumping compressed video buffer to disk.\n");
    write_compressed_video_file(file, xdisplay_video, xdisplay_video_pointer);

    return(1);
}

/*****************************************************************************/

int update_video_buffer(char *vid, int vid_ptr, int size, 
			char *frame,
			BOUNDING_BOX *b, int n)
{
  int i, tst;

  if(n > 0) {
    tst = sizeof(int);
    
    for(i = 0; i < n; i++)
      tst += compute_size_of_bounding_box(&b[i]);

    if(vid_ptr + tst < size) {
      memcpy(&vid[vid_ptr], &n, sizeof(int));
      vid_ptr += sizeof(int);
      
      for(i = 0; i < n; i++)
	vid_ptr += copy_bounding_box_to_char(&vid[vid_ptr], frame, &b[i]);
      
    } else {
      return(-1);
    }
  } else {
    if(vid_ptr + sizeof(int) < size) {
      memcpy(&vid[vid_ptr], &n, sizeof(int)); 
      vid_ptr += sizeof(int);
    } else {
      return(-1);
    }
  }
  
  return(vid_ptr);
}

/*****************************************************************************/

int interactive_compressed_video(char *vid, int n_vid, char *ref, char *curr,
				 FILE *fp)
{
  int done, advance, n_prev, vid_pointer = 0;
  int n_click_data = 0, i;
  POINT click_data[100];
  BOUNDING_BOX prev[N_BLOBS_MAX];
  XEvent event;

  done = 0;
  XSelectInput(xdisplay_Display, xdisplay_Window, ButtonPress);

  while(xdisplay_FrameCounter <= xdisplay_FrameMin)
    vid_pointer =
      play_compressed_video_one_frame(ref, curr, vid, vid_pointer, 
				      &prev[0], &n_prev, 0, 0);

  while(!done) {
    XWindowEvent(xdisplay_Display, xdisplay_Window, ButtonPress, &event);
    advance = handle_xdisplay_interactive_event(&event, 
						&click_data[0], 
						&n_click_data, fp);
    if(advance > 0) {
      vid_pointer =
	play_compressed_video_one_frame(ref, curr, vid, vid_pointer, 
					&prev[0], &n_prev, 0, 0);
      if(vid_pointer > n_vid)
	done = 1;
    } 
    
  }

  return(1);
}

/*****************************************************************************/

int handle_xdisplay_interactive_event(XEvent *e, POINT *click_data, int *n,
				      FILE *fp)
{
  int i, x, y, button, advance = 0;

  x = e->xbutton.x;
  y = e->xbutton.y;
  button = e->xbutton.button;

  if(button == Button1) {
    fprintf(stderr, "Frame #%d: Added pixel (%d, %d) [%d].\n", 
	    xdisplay_FrameCounter - 1, x, y, *n+1);
    click_data[*n].x = x;
    click_data[*n].y = y;
    (*n) = (*n) + 1;
    advance = 0;
  } else if(button == Button2) {
    fprintf(stderr, "Starting over for frame #%d...\n", 
	    xdisplay_FrameCounter - 1);
    *n = 0;
    advance = 0;
  } else {
    fprintf(stderr, "Advancing to frame #%d...\n", 
	    xdisplay_FrameCounter);
    fprintf(fp, "%d", xdisplay_FrameCounter - 1);
    for(i = 0; i < *n; i++) {
      fprintf(fp, " %d %d", click_data[i].x, click_data[i].y);
    }
    fprintf(fp, "\n");
    advance = 1;
    *n = 0;
  }

  return(advance);
}

/*****************************************************************************/

int extract_compressed_video(char *vid, int n_vid, char *ref, char *curr)
{
  BOUNDING_BOX prev[N_BLOBS_MAX];
  int n_prev = 0, vid_pointer = 0;
  int vid_first = -1, vid_last = -1;

  while(vid_pointer < n_vid) {
    if(xdisplay_FrameCounter == xdisplay_FrameMin) {
      vid_first = vid_pointer;
    }

    vid_pointer =
      play_compressed_video_one_frame(ref, curr, vid, vid_pointer, 
				      &prev[0], &n_prev, 0, 0);

    if(xdisplay_FrameCounter == xdisplay_FrameMax + 1) {
      vid_last = vid_pointer;
    }

  }

  if(vid_first < 0 || vid_last < 0) {
    fprintf(stderr, "Extract frame range is out of bounds. Exiting.\n");
    exit(-1);
  }

  write_compressed_video_file(&xdisplay_OutputFile[0], &vid[vid_first], 
			      vid_last - vid_first);
  return(1);
}

/*****************************************************************************/

int step_through_compressed_video(char *vid, int n_vid, char *ref, 
				  char *curr, int erase)
{
  int i, done, n_prev, vid_pointer = 0;
  int *vid_vec, first_frame;
  BOUNDING_BOX prev[N_BLOBS_MAX];
  int speed = 1, state = 1;
  XEvent event;

  done = 0;
  vid_vec = int_allocate_vector(vid_vec, MAX_XDISPLAY_STEP_FRAMES);
  first_frame = xdisplay_FrameCounter;
  vid_vec[xdisplay_FrameCounter-first_frame] = vid_pointer;

  XSelectInput(xdisplay_Display, xdisplay_Window, ButtonPress);
  vid_pointer =
    play_compressed_video_one_frame(ref, curr, vid, vid_pointer, 
				    &prev[0], &n_prev, 0, erase);
  vid_vec[xdisplay_FrameCounter-first_frame] = vid_pointer;

  while(!done) {
    XWindowEvent(xdisplay_Display, xdisplay_Window, ButtonPress, &event);
    state = handle_xdisplay_step_event(&event, &speed, state);

    switch(state)
      {
      case -1:
	{
	  xdisplay_FrameCounter-= ((speed)+1);
	  if(xdisplay_FrameCounter < first_frame)
	    xdisplay_FrameCounter = first_frame;

	  vid_pointer =
	    play_compressed_video_one_frame(ref, curr, vid, 
					    vid_vec[xdisplay_FrameCounter-
						   first_frame],
					    &prev[0], &n_prev, 0, erase);
	  
	  break;
	}
      case 0:
	{
	  xdisplay_FrameCounter--;
	  vid_pointer =
	    play_compressed_video_one_frame(ref, curr, vid, 
					    vid_vec[xdisplay_FrameCounter-
						   first_frame],
					    &prev[0], &n_prev, 0, erase);
	  
	  break;
	}
      case 1:
	{
	  for(i = 0; i < speed; i++) {
	    vid_pointer =
	      play_compressed_video_one_frame(ref, curr, vid, vid_pointer, 
					      &prev[0], &n_prev, 0, erase);
	    vid_vec[xdisplay_FrameCounter-first_frame] = vid_pointer;
	    if(vid_pointer > n_vid) {
	      xdisplay_FrameCounter-=2;
	      vid_pointer =
		play_compressed_video_one_frame(ref, curr, vid, 
						vid_vec[xdisplay_FrameCounter-
						       first_frame],
						&prev[0], &n_prev, 0, erase);
	    }
	  }
	  break;
	}
      }
  }

  free(vid_vec);
  return(1);
}

/*****************************************************************************/

int handle_xdisplay_step_event(XEvent *e, int *speed, int state)
{
  int button, advance = 0;

  button = e->xbutton.button;

  if(button == Button1) {
    if(state == 1) {
      *speed = 1;
      fprintf(stderr, "Reset speed = %d frames/click.\n", *speed);
    }
    advance = -1;
  } else if(button == Button2) {
    *speed = *speed + 1;
    advance = 0;
    fprintf(stderr, "Set speed = %d frames/click.\n", *speed);
  } else {
    if(state == -1) {
      *speed = 1;
      fprintf(stderr, "Reset speed = %d frames/click.\n", *speed);
    }
    advance = 1;
  }

  return(advance);
}

/*****************************************************************************/

int update_xdisplay_video_buffer_half(char *frame)
{
  int tst;

  tst = update_video_buffer_half(xdisplay_video, 
				 xdisplay_video_pointer,
				 xdisplay_video_buffer_size,
				 frame);
  if(tst > 0) {
    xdisplay_video_pointer = tst;
  } else {
    fprintf(stderr, "Video data is about to exceed maximum buffer size.\n");
    dump_xdisplay_video_buffer(&xdisplay_VideoOutputFile[0]);
    exit(1);
  }

  return(1);
}

/*****************************************************************************/

int update_video_buffer_half(char *vid, int vid_ptr, int size, char *frame)
{
  int i, tst, n;
  BOUNDING_BOX box;

  n = 1;
  box.xmin = 0;
  box.xmax = X_DIM/2 - 1;
  box.ymin = 0;
  box.ymax = Y_DIM/2 - 1;

  tst = sizeof(int);
  tst += compute_size_of_bounding_box(&box);

  if(vid_ptr + tst < size) {
    memcpy(&vid[vid_ptr], &n, sizeof(int));
    vid_ptr += sizeof(int);
      
    vid_ptr += copy_bounding_box_to_char_half(&vid[vid_ptr], frame, &box);
      
  } else {
    return(-1);
  }
  return(vid_ptr);
}

/*****************************************************************************/

int copy_bounding_box_to_char_half(char *vid, char *im, BOUNDING_BOX *b)
{
  char *buf;
  int i, j, loc, count = 0, int_length;

  int_length = sizeof(int);

  memcpy(&vid[count], &b->xmin, int_length); count += int_length;
  memcpy(&vid[count], &b->ymin, int_length); count += int_length;
  memcpy(&vid[count], &b->xmax, int_length); count += int_length;
  memcpy(&vid[count], &b->ymax, int_length); count += int_length;


  for(i = b->ymin; i < Y_DIM; i+=2) {
    for(j = b->xmin; j < X_DIM; j+=2) {
      loc = (i*X_DIM + j)*4;
      buf = &im[loc];
      vid[count++] = buf[0];
      vid[count++] = buf[1];
      vid[count++] = buf[2];
    }
  }

  return(count);
}

/*****************************************************************************/
